import numpy as np
import re


class ABCIDATA():
    ALLOCATE_SIZE = 10000

    def __init__(self):
        self.__cache_flush = True
        self.__data = {}
        self.__reverse = False
        self.__desc = {"title": "",
                       "xlab": "",
                       "ylab": "",
                       "else": []}
        self.__last_desc = "title"
        self.__store_to_new = False
        self.__last_data_name = ""
        self.__last_data_index = 0
        self.__legends = {}
        self.__pattern = re.compile(r"(X+|V+|F)")

    def __cache_allocate(self, n=ALLOCATE_SIZE):
        self.__cache = np.zeros((n, 2))
        self.__cache_pos = 0

    def __cache_crop(self):
        self.__cache.resize((self.__cache_pos, 2))

    def store_to_new(self, val):
        self.__store_to_new = val

    def insert(self, x, y):
        if self.__cache_flush:
            self.__cache_allocate()
            self.__cache_flush = False
        if len(self.__cache) == self.__cache_pos:
            self.__cache.resize((self.__cache_pos * 10, 2))
        self.__cache[self.__cache_pos] = [x, y]
        self.__cache_pos += 1

    def cache_dump(self):
        self.__cache_crop()
        return self.__cache

    def cache_store(self, name, new):
        self.__cache_crop()
        self.__last_data_name = name
        if name in self.__data:
            if self.__store_to_new or new:
                self.__data[name].append(self.__cache)
                self.store_to_new(False)
                self.__last_data_index = len(self.__data[name]) - 1
            else:
                self.__data[name][0] = np.concatenate((self.__data[name][0],
                                                       self.__cache))
                self.__last_data_index = 0
        else:
            self.__data[name] = [self.__cache]
            self.__last_data_index = 0
        self.__cache_flush = True

    def last_is_legend(self, text):
        self.__legends[self.__last_data_name] = text
        self.__data[self.__last_data_name].pop(self.__last_data_index)

    def legends_dump(self):
        return self.__legends

    def data_dump(self):
        return self.__data

    def title_add(self, text):
        self.__desc["title"] = text
        self.__last_desc = "title"

    def title_dump(self):
        return self.__desc["title"].strip()

    def xlab_add(self, text):
        self.__desc["xlab"] = text
        self.__last_desc = "xlab"

    def xlab_dump(self):
        return self.__desc["xlab"].strip()

    def ylab_add(self, text):
        self.__desc["ylab"] = text
        self.__last_desc = "ylab"

    def ylab_dump(self):
        return self.__desc["ylab"].strip()

    def apply_case(self, overlay):
        if self.__last_desc == "else":
            strings = self.__desc[self.__last_desc]
            string = strings[len(strings) - 1]
        else:
            string = self.__desc[self.__last_desc]
        match = [m for m in re.finditer(self.__pattern, overlay)]
        match.reverse()
        for m in match:
            if "X" in m.group(0):
                string = string[:m.start()] + \
                         "$^{" + string[m.start():m.end()] + \
                         "}$" + string[m.end():]
            if "V" in m.group(0):
                string = string[:m.start()] + \
                         "$_{" + string[m.start():m.end()] + \
                         "}$" + string[m.end():]
            if "F" in m.group(0):
                if string[m.start()] == "W":
                    string = string[:m.start()] + \
                        r"$\Omega$" + string[m.end():]
        if self.__last_desc == "else":
            self.__desc[self.__last_desc][len(strings) - 1] = string
        else:
            self.__desc[self.__last_desc] = string

    def append_last(self, text):
        if self.__last_desc == "else":
            self.__desc["else"][len(self.__desc["else"])-1] = \
                self.__desc["else"][len(self.__desc["else"])-1] + text
        else:
            self.__desc[self.__last_desc] = \
                self.__desc[self.__last_desc] + text

    def description_add(self, text):
        self.__desc["else"].append(text)
        self.__last_desc = "else"

    def description_dump(self):
        return self.__desc["else"]

    def reverse(self, val=None):
        if val is not None:
            self.__reverse = val
        return self.__reverse


def top_parse(f):
    buf = []
    cur = None
    # dig = re.compile(r"(\d+\.\d+|\d+\.?|\.?\d+|NaN)")
    try:
        with open(f) as text:
            newpart = True
            for line in text:
                linel = line.strip().split()
                if cur is not None and len(linel) == 2:
                    try:
                        cur.insert(float(linel[0]), float(linel[1]))
                        continue
                    except ValueError:
                        pass  # fallthrough
                if cur is not None and linel[0] == "SET":
                    if len(linel) > 3 and linel[1] == "ORDER":
                        if linel[2] == "X":
                            cur.reverse(True)
                    continue
                if cur is not None and linel[0] == "TITLE":
                    cur.store_to_new(True)
                    li = line.strip().split("'")
                    li = [i for i in li if len(i) > 0]
                    msg = ""
                    if len(li) > 1:
                        msg = msg.join(li[1:])
                    if len(linel) > 3 and newpart:
                        cur.title_add(msg)
                    elif "BOTTOM" in linel:
                        cur.xlab_add(msg)
                    elif "LEFT" in linel:
                        cur.ylab_add(msg)
                    elif "DATA" in linel and "SIZE" in linel:
                        cur.last_is_legend(msg)
                    else:
                        cur.description_add(msg)
                    newpart = False
                    continue
                if cur is not None and linel[0] == "MORE":
                    li = line.strip().split("'")
                    li = [i for i in li if len(i) > 0]
                    msg = ""
                    if len(li) > 1:
                        msg = msg.join(li[1:]).strip()
                    cur.append_last(msg)
                    continue
                if cur is not None and linel[0] == "CASE":
                    li = line.strip().split("'")
                    li = [i for i in li if len(i) > 0]
                    msg = ""
                    if len(li) > 1:
                        msg = msg.join(li[1:])
                    cur.apply_case(msg)
                    continue
                if cur is not None and linel[0] == "JOIN":
                    new = True
                    if len(linel) > 1 and linel[1].isdigit():
                        new = True
                    if "DOTS" in linel:
                        cur.cache_store("dotted", new)
                    elif "DASH" in linel:
                        cur.cache_store("dashed", new)
                    elif "DOTDASH" in linel:
                        cur.cache_store("dashdot", new)
                    else:
                        cur.cache_store("solid", new)
                    continue
                if linel[0] == "NEW":
                    if cur is not None:
                        buf.append(cur)
                    cur = ABCIDATA()
                    newpart = True
                    continue
                if linel[0] == "PLOT":
                    continue
            if cur is not None:
                buf.append(cur)
    except PermissionError:
        pass

    return(buf)



from sys import argv, exit
from os.path import dirname
from PyQt5 import QtWidgets
from pytopplot.ui.window import MainWindow


def main():
    app = QtWidgets.QApplication(argv)
    window = MainWindow(dirname(__file__))
    exit(app.exec_())



SETTINGS_VERSION = 2

DEFAULTGEN = {"TITLE": "Project Title",
              "LSAV": False,
              "ITEST": 1,
              "LREC": False,
              "LCPUTM": False,
              "TSOS": 5,
              "TMAX": 3600, }

DEFAULTTIME = {"MT": 3,
               "NSHOT": 5,
               "TPS": None, }

DEFAULTMESH = {"DDR": 0.001,
               "DDZ": 0.001,
               "NR": None,
               "NZ": None,
               "RMARK": None,
               "RZ": None, }

DEFAULTBEAM = {"MROT": 0,
               "SIG": None,
               "ISIG": 5,
               "RDRIVE": None,
               "NBUNCH": 1,
               "BSEP": 0, }

DEFAULTWAKE = {"UBT": None,
               "LCFRON": True,
               "LCBACK": True,
               "LCHIN": True,
               "LNAPOLY": False,
               "LNONAP": False,
               "LCRBW": False,
               "ZCF": None,
               "ZCT": None,
               "ZSEP": 0,
               "RWAK": None, }

DEFAULTPLOT = {"LCAVIN": True,
               "LCAVUS": False,
               "LPATH": False,
               "LPLW": False,
               "LPLWL": False,
               "LPLWA": False,
               "LPLWT": False,
               "LFFT": False,
               "LFFTL": False,
               "LFFTA": False,
               "LFFTT": False,
               "LINTZ": False,
               "LSPEC": False,
               "LWNDW": False,
               "NWFUN": 1,
               "ALPHA": 3,
               "CUTOFF": 0,
               "EXPFAC": 20,
               "LPLE": False,
               "LPLC": False,
               "NPLOT": 10,
               "LPALL": False, }

DEFAULTPRINT = {"LPRW": False,
                "LMATPR": False,
                "LSVW": False,
                "LSVWL": False,
                "LSVWA": False,
                "LSVWT": False,
                "LSVF": False, }


class JOB():
    def __init__(self, name=None, index=None, settings=None):
        self.__name = name
        self.__index = index
        self.__settings = settings

    def name(self, name=None):
        if name is not None:
            self.__name = name
        else:
            return self.__name

    def index(self, index=None):
        if index is not None:
            self.__index = index
        else:
            return self.__index

    def settings(self, settings=None):
        if settings is not None:
            self.__settings = settings
        else:
            return self.__settings


class JobList():
    def __init__(self):
        self.__VERSION = SETTINGS_VERSION
        self.__common = {"gensec": DEFAULTGEN,
                         "timesec": DEFAULTTIME, }

        self.__anonym = {"meshsec": DEFAULTMESH,
                         "beamsec": DEFAULTBEAM,
                         "wakesec": DEFAULTWAKE,
                         "plotsec": DEFAULTPLOT,
                         "printsec": DEFAULTPRINT, }

        self.__list = []

    def add(self, name, index, settings):
        self.pop(index)
        j = JOB(name=name, index=index, settings=settings)
        self.__list.append(j)
        self.__list.sort(key=lambda x: x.index())

    def push(self, obj):
        self.__list.append(obj)
        self.__list.sort(key=lambda x: x.index())

    def pop(self, index):
        try:
            i = [i.index() for i in self.__list].index(index)
            return self.__list.pop(i)
        except ValueError:
            return None

    def get(self, index):
        try:
            i = [i.index() for i in self.__list].index(index)
            return self.__list[i]
        except ValueError:
            return None

    def set_anonym(self, settings):
        self.__anonym = settings

    def get_anonym(self):
        return self.__anonym

    def set_common(self, settings):
        self.__common = settings

    def get_common(self):
        return self.__common

    def dump_settings(self):
        if len(self.__list) == 0:
            return [self.get_anonym()]
        else:
            return [i.settings() for i in self.__list]

    def iterate_list(self):
        for i in self.__list:
            yield i

    def version(self):
        return self.__VERSION


def generate_file(file, geomtype, geom, commsec, jobsec):
    B = {True: ".T.",
         False: ".F.",
         None: "",
         "Absolute": "##",
         "Incremental": "@@"}
    try:
        with open(file, "w") as f:
            # General options
            f.write(f""" &FILE LSAV={B[commsec["gensec"]["LSAV"]]}, \
ITEST={commsec["gensec"]["ITEST"]}, LREC={B[commsec["gensec"]["LSAV"]]}, \
LCPUTM={B[commsec["gensec"]["LSAV"]]}, TSOS={commsec["gensec"]["TSOS"]}, \
TMAX={commsec["gensec"]["TMAX"]} &END\n""")
            # Title
            f.write(f""" {commsec["gensec"]["TITLE"]}\n""")
            # Dummy
            f.write(" &BOUN IZL=3, IZR=3 &END\n")
            # Mesh
            f.write(f""" &MESH DDZ={jobsec[0]["meshsec"]["DDZ"]}""")
            if jobsec[0]["meshsec"]["DDR"] is not None:
                f.write(f""", DDR={jobsec[0]["meshsec"]["DDR"]}""")
            if jobsec[0]["meshsec"]["NR"] is not None:
                f.write(f""", NR={jobsec[0]["meshsec"]["NR"]}""")
            if jobsec[0]["meshsec"]["NZ"] is not None:
                f.write(f""", NZ={jobsec[0]["meshsec"]["NZ"]}""")
            if jobsec[0]["meshsec"]["RMARK"] is not None:
                f.write(f""", RMARK={jobsec[0]["meshsec"]["RMARK"]}""")
            if jobsec[0]["meshsec"]["RZ"] is not None:
                f.write(f""", RZ={jobsec[0]["meshsec"]["RZ"]}""")
            f.write(" &END\n")
            # Geometry points
            f.write(f""" {B[geomtype]}CAVITYSHAPE\n""")
            f.write("0.\n")
            for r, z in geom:
                f.write(f"{r} {z}\n")
            f.write("9999. 9999.\n")
            firstpass = True
            for j in jobsec:
                if not firstpass:
                    f.write("CONTINUE\n")
                    # Mesh
                    f.write(f""" &MESH""")
                    if j["meshsec"]["DDZ"] is not None:
                        f.write(f""" DDZ={j["meshsec"]["DDZ"]}""")
                    else:
                        f.write(f""" DDZ=0.001""")
                    if j["meshsec"]["DDR"] is not None:
                        f.write(f""", DDR={j["meshsec"]["DDR"]}""")
                    else:
                        f.write(f""", DDR=0.001""")
                    if j["meshsec"]["NR"] is not None:
                        f.write(f""", NR={j["meshsec"]["NR"]}""")
                    if j["meshsec"]["NZ"] is not None:
                        f.write(f""", NZ={j["meshsec"]["NZ"]}""")
                    if j["meshsec"]["RMARK"] is not None:
                        f.write(f""", RMARK={j["meshsec"]["RMARK"]}""")
                    if j["meshsec"]["RZ"] is not None:
                        f.write(f""", RZ={j["meshsec"]["RZ"]}""")
                    f.write(" &END\n")
                # Beam
                f.write(f""" &BEAM MROT={j["beamsec"]["MROT"]}, \
ISIG={j["beamsec"]["ISIG"]}, NBUNCH={j["beamsec"]["NBUNCH"]}, \
BSEP={j["beamsec"]["BSEP"]}""")
                if j["beamsec"]["SIG"] is not None:
                    f.write(f""", SIG={j["beamsec"]["SIG"]}""")
                if j["beamsec"]["RDRIVE"] is not None:
                    f.write(f""", RDRIVE={j["beamsec"]["RDRIVE"]}""")
                f.write(" &END\n")
                if firstpass:
                    # Time
                    f.write(f""" &TIME MT={commsec["timesec"]["MT"]}, \
NSHOT={commsec["timesec"]["NSHOT"]}""")
                    if commsec["timesec"]["TPS"] is not None:
                        f.write(f""", TPS={commsec["timesec"]["TSP"]}""")
                    f.write(" &END\n")
                # Wake
                f.write(f""" &WAKE LCFRON={j["wakesec"]["LCFRON"]}, \
LCBACK={j["wakesec"]["LCBACK"]}, LCHIN={j["wakesec"]["LCHIN"]}, \
LNAPOLY={j["wakesec"]["LNAPOLY"]}, LNONAP={j["wakesec"]["LNONAP"]}, \
LCRBW={j["wakesec"]["LCRBW"]}, ZSEP={j["wakesec"]["ZSEP"]}""")
                if j["wakesec"]["UBT"] is not None:
                    f.write(f""", UBT={j["wakesec"]["UBT"]}""")
                if j["wakesec"]["ZCF"] is not None:
                    f.write(f""", ZCF={j["wakesec"]["ZCF"]}""")
                if j["wakesec"]["ZCT"] is not None:
                    f.write(f""", ZCT={j["wakesec"]["ZCT"]}""")
                if j["wakesec"]["RWAK"] is not None:
                    f.write(f""", RWAK={j["wakesec"]["RWAK"]}""")
                f.write(" &END\n")
                # Plot
                f.write(f""" &PLOT LCAVIN={j["plotsec"]["LCAVIN"]}, \
LCAVUS={j["plotsec"]["LCAVUS"]}, LPATH={j["plotsec"]["LPATH"]}, \
LPLW={j["plotsec"]["LPLW"]}, LPLWL={j["plotsec"]["LPLWL"]}, \
LPLWA={j["plotsec"]["LPLWA"]}, LPLWT={j["plotsec"]["LPLWT"]}, \
LFFT={j["plotsec"]["LFFT"]}, LFFTA={j["plotsec"]["LFFTA"]}, \
LFFTA={j["plotsec"]["LFFTA"]}, LFFTT={j["plotsec"]["LFFTT"]}, \
LINTZ={j["plotsec"]["LINTZ"]}, LWNDW={j["plotsec"]["LWNDW"]}, \
NWFUN={j["plotsec"]["NWFUN"]}, ALPHA={j["plotsec"]["ALPHA"]}, \
CUTOFF={j["plotsec"]["CUTOFF"]}, EXPFAC={j["plotsec"]["EXPFAC"]}, \
LPLE={j["plotsec"]["LPLE"]}, LPLC={j["plotsec"]["LPLC"]}, \
NPLOT={j["plotsec"]["NPLOT"]}, LPALL={j["plotsec"]["LPALL"]} &END\n""")
                # Print
                f.write(f""" &PRIN LPRW={j["printsec"]["LPRW"]}, \
LMATPR={j["printsec"]["LMATPR"]}, LSVW={j["printsec"]["LSVW"]}, \
LSVWL={j["printsec"]["LSVWL"]}, LSVWA={j["printsec"]["LSVWA"]}, \
LSVWT={j["printsec"]["LSVWT"]}, LSVF={j["printsec"]["LSVF"]} &END\n""")
                firstpass = False
            f.write("STOP\n")
    except PermissionError:
        pass



from tempfile import mkdtemp
from os import path
from time import sleep
import subprocess
import csv
from shutil import rmtree, copyfile
from os import environ, access, X_OK
from os.path import isfile
import platform
import yaml

from PyQt5 import QtWidgets, uic, QtGui, QtCore
from matplotlib.backends.backend_qt5agg \
    import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg \
    import NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure
import matplotlib.lines as mlines

import pytopplot.generator.generator as gen
import pytopplot.parse.parser as prs

ITEST = {"Calculate": 0, "Test": 1, }
MROT = {"Monopole fields": 0, "Dipole fields": 1, }
NWFUN = {"Kaiser-Bessel window": 1,
         "4-term 74dB Blackman-Harris window": 2,
         "Gaussian window": 3, }
RITEST = {j: i for i, j in ITEST.items()}
RMROT = {j: i for i, j in MROT.items()}
RNWFUN = {j: i for i, j in NWFUN.items()}


class MainWindow(QtWidgets.QMainWindow):
    def __init__(self, topdir):
        super(MainWindow, self).__init__()
        uic.loadUi(path.join(topdir,
                             "ui",
                             "window.ui"), self)
        self.appCloseAction.triggered.connect(self.close)
        self.execLocation.setText(path.join(topdir,
                                            "abci",
                                            platform.system(),
                                            "abci.exe"))
        self.tempDir = None
        self.data = []
        self.topdir = topdir
        self.lastdir = environ["HOMEPATH"] if \
            platform.system() == "Windows" else environ["HOME"]

        # setup matplotlib
        self.pCanvas = MplCanvas(self)
        self.pToolbar = NavigationToolbar(self.pCanvas, self)
        self.pFrame.layout().addWidget(self.pCanvas)
        self.pFrame.layout().addWidget(self.pToolbar)

        # geometry related
        self.gCanvas = MplCanvas(self)
        self.gFrame.layout().addWidget(self.gCanvas)
        self.gToolbar = NavigationToolbar(self.gCanvas, self)
        self.gFrame.layout().addWidget(self.gToolbar)
        self.gUploadButton.clicked.connect(self.gupload)
        self.gDownloadButton.clicked.connect(self.gdownload)
        self.gDrawButton.clicked.connect(self.gdraw)
        self.gStyle.addItems(["Absolute", "Incremental"])
        self.gMenu = QtWidgets.QMenu()
        self.gTable.contextMenuEvent = \
            lambda event: self.gMenu.popup(QtGui.QCursor.pos())
        gAddBAction = QtWidgets.QAction("Add row before", self)
        gAddBAction.triggered.connect(self.gaddb)
        self.gMenu.addAction(gAddBAction)
        gAddAAction = QtWidgets.QAction("Add row after", self)
        gAddAAction.triggered.connect(self.gadda)
        self.gMenu.addAction(gAddAAction)
        gRemAction = QtWidgets.QAction("Remove Rows", self)
        gRemAction.triggered.connect(self.grem)
        self.gMenu.addAction(gRemAction)

        # job related
        self.jABCUploadButton.clicked.connect(self.jabcupload)
        self.jABCDownloadButton.clicked.connect(self.jabcdownload)
        self.jYAMLUploadButton.clicked.connect(self.jyamlupload)
        self.jYAMLDownloadButton.clicked.connect(self.jyamldownload)
        self.jCalculateButton.clicked.connect(self.jcalculate)
        self.ITEST.addItems(["Calculate", "Test"])
        self.MROT.addItems(["Monopole fields", "Dipole fields"])
        self.NWFUN.addItems(["Kaiser-Bessel window",
                             "4-term 74dB Blackman-Harris window",
                             "Gaussian window"])
        self.jTable.itemSelectionChanged.connect(self.jtableselchanged)
        self.jTable.itemChanged.connect(self.jtableitemchanged)
        self.jMenu = QtWidgets.QMenu()
        self.jTable.contextMenuEvent = \
            lambda event: self.jMenu.popup(QtGui.QCursor.pos())
        jAddBAction = QtWidgets.QAction("Add row before", self)
        jAddBAction.triggered.connect(self.jaddb)
        self.jMenu.addAction(jAddBAction)
        jAddAAction = QtWidgets.QAction("Add row after", self)
        jAddAAction.triggered.connect(self.jadda)
        self.jMenu.addAction(jAddAAction)
        jRemAction = QtWidgets.QAction("Remove Row", self)
        jRemAction.triggered.connect(self.jrem)
        self.jMenu.addAction(jRemAction)
        self.LCBACK.toggled.connect(self.jlcbackchanged)
        self.LPLE.toggled.connect(self.jlplchanged)
        self.LPLC.toggled.connect(self.jlplchanged)

        # draw related
        self.pList.currentRowChanged.connect(self.plistselect)
        self.pUploadButton.clicked.connect(self.pupload)
        self.pDownloadButton.clicked.connect(self.pdownload)

        # settings
        self.execFindButton.clicked.connect(self.ssetExec)

        # shortcuts
        self.gShortcut = QtWidgets.QShortcut(QtGui.QKeySequence("Ctrl+1"),
                                             self)
        self.gShortcut.activated.connect(lambda: self.Tabs.setCurrentIndex(0))
        self.jShortcut = QtWidgets.QShortcut(QtGui.QKeySequence("Ctrl+2"),
                                             self)
        self.jShortcut.activated.connect(lambda: self.Tabs.setCurrentIndex(1))
        self.pShortcut = QtWidgets.QShortcut(QtGui.QKeySequence("Ctrl+3"),
                                             self)
        self.pShortcut.activated.connect(lambda: self.Tabs.setCurrentIndex(2))
        self.sShortcut = QtWidgets.QShortcut(QtGui.QKeySequence("Ctrl+4"),
                                             self)
        self.sShortcut.activated.connect(lambda: self.Tabs.setCurrentIndex(3))
        self.sShortcut = QtWidgets.QShortcut(QtGui.QKeySequence("Ctrl+PgDown"),
                                             self)
        self.sShortcut.activated.connect(self.nexttab)
        self.sShortcut = QtWidgets.QShortcut(QtGui.QKeySequence("Ctrl+PgUp"),
                                             self)
        self.sShortcut.activated.connect(self.prevtab)
        self.genShortcut = QtWidgets.QShortcut(QtGui.QKeySequence("g"),
                                               self)
        self.genShortcut.activated.connect(self.gentab)
        self.meshShortcut = QtWidgets.QShortcut(QtGui.QKeySequence("m"),
                                                self)
        self.meshShortcut.activated.connect(self.meshtab)
        self.beamShortcut = QtWidgets.QShortcut(QtGui.QKeySequence("b"),
                                                self)
        self.beamShortcut.activated.connect(self.beamtab)
        self.timeShortcut = QtWidgets.QShortcut(QtGui.QKeySequence("t"),
                                                self)
        self.timeShortcut.activated.connect(self.timetab)
        self.wakeShortcut = QtWidgets.QShortcut(QtGui.QKeySequence("w"),
                                                self)
        self.wakeShortcut.activated.connect(self.waketab)
        self.plotShortcut = QtWidgets.QShortcut(QtGui.QKeySequence("p"),
                                                self)
        self.plotShortcut.activated.connect(self.plottab)
        self.printShortcut = QtWidgets.QShortcut(QtGui.QKeySequence("l"),
                                                 self)
        self.printShortcut.activated.connect(self.printtab)

        self.jobList = gen.JobList()
        self.jobLastSelected = -1
        self.show()

    def gentab(self, *arg):
        self.settingsPage.setCurrentIndex(0)

    def meshtab(self, *arg):
        self.settingsPage.setCurrentIndex(1)

    def beamtab(self, *arg):
        self.settingsPage.setCurrentIndex(2)

    def timetab(self, *arg):
        self.settingsPage.setCurrentIndex(3)

    def waketab(self, *arg):
        self.settingsPage.setCurrentIndex(4)

    def plottab(self, *arg):
        self.settingsPage.setCurrentIndex(5)

    def printtab(self, *arg):
        self.settingsPage.setCurrentIndex(6)

    def nexttab(self, *arg):
        next = self.Tabs.currentIndex() + 1 if self.Tabs.currentIndex() \
               < self.Tabs.count() - 1 else 0
        self.Tabs.setCurrentIndex(next)

    def prevtab(self, *arg):
        next = self.Tabs.currentIndex() - 1 if self.Tabs.currentIndex() \
               > 0 else self.Tabs.count() - 1
        self.Tabs.setCurrentIndex(next)

    def gupload(self, *arg):
        file, type = QtWidgets.QFileDialog.getOpenFileName(
            self,
            "Open File",
            self.lastdir,
            "CSV Files (*.csv);;All Files (*)"
        )
        if isfile(file):
            self.lastdir = path.dirname(file)
            try:
                with open(file, "r") as f:
                    self.gTable.setRowCount(0)
                    reader = csv.reader(f)
                    i = 0
                    for row in reader:
                        if len(row) == 2:
                            self.gTable.insertRow(i)
                            r = QtWidgets.QTableWidgetItem()
                            r.setText(row[0])
                            z = QtWidgets.QTableWidgetItem()
                            z.setText(row[1])
                            self.gTable.setItem(i, 0, r)
                            self.gTable.setItem(i, 1, z)
                            i += 1
            except PermissionError:
                dialog = AlertWindow(self.topdir, "Cannot open file")
                dialog.exec_()

    def gdownload(self, *arg):
        file, type = QtWidgets.QFileDialog.getSaveFileName(
            self,
            "Save File",
            self.lastdir,
            "CSV Files (*.csv);;All Files (*)"
        )
        if file != "":
            self.lastdir = path.dirname(file)
            try:
                with open(file, "w") as f:
                    wr = csv.writer(f, quoting=csv.QUOTE_ALL)
                    for i in range(0, self.gTable.rowCount()):
                        r = ""
                        z = ""
                        if self.gTable.item(i, 0) is not None and \
                           self.gTable.item(i, 0).text() != "":
                            r = self.gTable.item(i, 0).text()
                            if self.gTable.item(i, 1) is not None and \
                               self.gTable.item(i, 1).text() != "":
                                z = self.gTable.item(i, 1).text()
                                if r != "" or z != "":
                                    wr.writerow((r, z))
            except PermissionError:
                dialog = AlertWindow(self.topdir, "Cannot open file")
                dialog.exec_()

    def gdraw(self, *arg):
        data = []
        for i in range(0, self.gTable.rowCount()):
            r = ""
            z = ""
            if self.gTable.item(i, 0) is not None and \
               self.gTable.item(i, 0).text() != "":
                r = self.gTable.item(i, 0).text()
            if self.gTable.item(i, 1) is not None and \
               self.gTable.item(i, 1).text() != "":
                z = self.gTable.item(i, 1).text()
            if r != "" or z != "":
                data.append((r, z))

        if self.tempDir is not None:
            try:
                rmtree(self.tempDir)
            except FileNotFoundError:
                pass
        self.tempDir = mkdtemp()
        inputfile = path.join(self.tempDir, "workfile.abc")
        outputfile = path.join(self.tempDir, "workfile.top")
        gen.generate_file(inputfile,
                          self.gStyle.currentText(),
                          data,
                          {"gensec": gen.DEFAULTGEN,
                           "timesec": gen.DEFAULTTIME, },
                          [{"meshsec": gen.DEFAULTMESH,
                            "beamsec": gen.DEFAULTBEAM,
                            "wakesec": gen.DEFAULTWAKE,
                            "plotsec": gen.DEFAULTPLOT,
                            "printsec": gen.DEFAULTPRINT, }])
        proc = subprocess.Popen([self.execLocation.text(), inputfile],
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
        rv = proc.wait()
        if rv != 0:
            out = "\n".join([l.decode().strip()
                             for l in proc.stderr.readlines()])
            out = "Non-zero exit code\n" + out
            dialog = AlertWindow(self.topdir, out)
            dialog.exec_()
        else:
            plto = prs.top_parse(outputfile)
            if len(plto) == 0:
                dialog = AlertWindow(self.topdir, "Top parser error. \
Check geometry.")
                dialog.exec_()
            else:
                plto = plto[0]
                data = plto.data_dump()
                self.gCanvas.axes.clear()
                for style in data.keys():
                    for d in data[style]:
                        self.gCanvas.axes.plot(d[:, 0],
                                               d[:, 1],
                                               linestyle=style,
                                               color="black")
                        self.gCanvas.axes.set_title(plto.title_dump())
                        self.gCanvas.axes.set_xlabel(plto.xlab_dump())
                self.gCanvas.axes.set_ylabel(plto.ylab_dump())
                self.gCanvas.axes.grid()
                legends = []
                for style, label in plto.legends_dump().items():
                    legends.append(mlines.Line2D([], [],
                                                 color='black',
                                                 linestyle=style,
                                                 label=label))
                if len(legends) > 0:
                    self.gCanvas.axes.legend(handles=legends)
                self.gCanvas.draw()

    def gaddb(self, *arg):
        self.gTable.insertRow(self.gTable.currentRow())

    def gadda(self, *arg):
        self.gTable.insertRow(self.gTable.currentRow() + 1)

    def grem(self, *arg):
        rmrange = self.gTable.selectedRanges()[0]
        for i in range(rmrange.bottomRow(), rmrange.topRow() - 1, -1):
            self.gTable.removeRow(i)

    def jabcupload(self, *arg):
        file, type = QtWidgets.QFileDialog.getOpenFileName(
                self,
                "Open File",
                self.lastdir,
                "ABC Files (*.abc);;All Files (*)"
        )
        if isfile(file):
            self.Tabs.setEnabled(False)
            self.Tabs.repaint()

            if self.tempDir is not None:
                try:
                    rmtree(self.tempDir)
                except FileNotFoundError:
                    pass
            self.tempDir = mkdtemp()
            inputfile = path.join(self.tempDir, "workfile.abc")
            copyfile(file, inputfile)
            outputfile = path.join(self.tempDir, "workfile.top")
            proc = subprocess.Popen([self.execLocation.text(), inputfile],
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE,
                                    bufsize=0)
            progress = ProgressWindow(self.topdir, proc)
            progress.exec_()
            rv = proc.wait()
            if rv != 0:
                out = "\n".join([l.decode().strip()
                                 for l in proc.stderr.readlines()])
                out = "Non-zero exit code\n" + out
                dialog = AlertWindow(self.topdir, out)
                dialog.exec_()
            else:
                plto = prs.top_parse(outputfile)
                if len(plto) == 0:
                    dialog = AlertWindow(self.topdir, "Top parser error. \
    Check geometry and settings.")
                    dialog.exec_()
                else:
                    self.data = plto
                    self.pList.clear()
                    for i in self.data:
                        title = i.title_dump() if i.title_dump() != "" \
                            else "No name avaliable"
                        self.pList.addItem(title)

            self.Tabs.setEnabled(True)

    def jabcdownload(self, *arg):
        file, type = QtWidgets.QFileDialog.getSaveFileName(
            self,
            "Save File",
            self.lastdir,
            "ABC Files (*.abc);;All Files (*)"
        )
        if file != "":
            j = self.jobList.get(self.jobLastSelected)
            if j is not None:
                j.settings(self.jgetsettings())
            else:
                self.jobList.set_anonym(self.jgetsettings())
            self.jobList.set_common(self.jgetcommsettings())
            data = []
            for i in range(0, self.gTable.rowCount()):
                r = ""
                z = ""
                if self.gTable.item(i, 0) is not None and \
                   self.gTable.item(i, 0).text() != "":
                    r = self.gTable.item(i, 0).text()
                    if self.gTable.item(i, 1) is not None and \
                       self.gTable.item(i, 1).text() != "":
                        z = self.gTable.item(i, 1).text()
                    if r != "" or z != "":
                        data.append((r, z))

                if self.tempDir is not None:
                    try:
                        rmtree(self.tempDir)
                    except FileNotFoundError:
                        pass
                self.tempDir = mkdtemp()
                inputfile = path.join(self.tempDir, "workfile.abc")
                gen.generate_file(inputfile,
                                  self.gStyle.currentText(),
                                  data,
                                  self.jobList.get_common(),
                                  self.jobList.dump_settings())
            try:
                copyfile(inputfile, file)
            except IOError:
                dialog = AlertWindow(self.topdir, "Cannot open file")
                dialog.exec_()

    def jyamlupload(self, *arg):
        file, type = QtWidgets.QFileDialog.getOpenFileName(
            self,
            "Open File",
            self.lastdir,
            "YAML Files (*.yaml);;All Files (*)"
        )
        if file != "":
            self.lastdir = path.dirname(file)
            try:
                with open(file, "r") as f:
                    jobList = yaml.load(f)
                    if jobList.version() != gen.SETTINGS_VERSION:
                        dialog = AlertWindow(self.topdir,
                                             "File version do not match")
                        dialog.exec_()
                    else:
                        self.jobList = jobList
                        self.jTable.blockSignals(True)
                        self.jTable.setRowCount(0)
                        i = 0
                        for j in self.jobList.iterate_list():
                            new = QtWidgets.QTableWidgetItem()
                            new.setText(j.name())
                            j.index(i)
                            self.jTable.insertRow(i)
                            self.jTable.setItem(i, 0, new)
                            i+=1
                        new = QtWidgets.QTableWidgetItem()
                        self.jTable.insertRow(i)
                        self.jTable.setItem(i, 0, new)
                        self.jTable.setCurrentCell(0, 0)
                        self.jobLastSelected = 0
                        j = self.jobList.get(0)
                        if j is None:
                            self.jsetsettings(self.jobList.get_anonym())
                        else:
                            self.jsetsettings(j.settings())
                        self.jsetcommsettings(self.jobList.get_common())
                        self.jTable.blockSignals(False)
            except PermissionError:
                dialog = AlertWindow(self.topdir, "Cannot open file")
                dialog.exec_()

    def jyamldownload(self, *arg):
        file, type = QtWidgets.QFileDialog.getSaveFileName(
            self,
            "Save File",
            self.lastdir,
            "YAML Files (*.yaml);;All Files (*)"
        )
        if file != "":
            self.lastdir = path.dirname(file)
            self.jobList.set_common(self.jgetcommsettings())
            j = self.jobList.get(self.jobLastSelected)
            if j is None:
                self.jobList.set_anonym(self.jgetsettings())
            else:
                j.settings(self.jgetsettings())
            try:
                with open(file, "w") as f:
                    yaml.dump(self.jobList, f)
            except PermissionError:
                dialog = AlertWindow(self.topdir, "Cannot open file")
                dialog.exec_()

    def jlcbackchanged(self, *arg):
        if arg[0]:
            self.LPLE.setChecked(False)
            self.LPLC.setChecked(False)

    def jlplchanged(self, *arg):
        if arg[0]:
            self.LCBACK.setChecked(False)

    def jcalculate(self, *arg):
        self.Tabs.setEnabled(False)
        self.Tabs.repaint()
        j = self.jobList.get(self.jobLastSelected)
        if j is None:
            self.jobList.set_anonym(self.jgetsettings())
        else:
            j.settings(self.jgetsettings())
        self.jobList.set_common(self.jgetcommsettings())
        data = []
        for i in range(0, self.gTable.rowCount()):
            r = ""
            z = ""
            if self.gTable.item(i, 0) is not None and \
               self.gTable.item(i, 0).text() != "":
                r = self.gTable.item(i, 0).text()
            if self.gTable.item(i, 1) is not None and \
               self.gTable.item(i, 1).text() != "":
                z = self.gTable.item(i, 1).text()
            if r != "" or z != "":
                data.append((r, z))

        if self.tempDir is not None:
            try:
                rmtree(self.tempDir)
            except FileNotFoundError:
                pass
        self.tempDir = mkdtemp()
        inputfile = path.join(self.tempDir, "workfile.abc")
        outputfile = path.join(self.tempDir, "workfile.top")
        gen.generate_file(inputfile,
                          self.gStyle.currentText(),
                          data,
                          self.jobList.get_common(),
                          self.jobList.dump_settings())
        proc = subprocess.Popen([self.execLocation.text(), inputfile],
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE,
                                bufsize=0)
        progress = ProgressWindow(self.topdir, proc)
        progress.exec_()
        rv = proc.wait()
        if rv != 0:
            out = "\n".join([l.decode().strip()
                             for l in proc.stderr.readlines()])
            out = "Non-zero exit code\n" + out
            dialog = AlertWindow(self.topdir, out)
            dialog.exec_()
        else:
            plto = prs.top_parse(outputfile)
            if len(plto) == 0:
                dialog = AlertWindow(self.topdir, "Top parser error. \
Check geometry and settings.")
                dialog.exec_()
            else:
                self.data = plto
                self.pList.clear()
                for i in self.data:
                    title = i.title_dump() if i.title_dump() != "" \
                        else "No name avaliable"
                    self.pList.addItem(title)
        self.Tabs.setEnabled(True)

    def jaddb(self, *arg):
        self.jTable.blockSignals(True)
        index = self.jTable.currentRow()
        self.jTable.insertRow(index)
        [j.index(j.index() + 1) for j in self.jobList.iterate_list()
         if j.index() >= index]
        self.jobLastSelected = self.jobLastSelected + 1
        self.jTable.blockSignals(False)

    def jadda(self, *arg):
        self.jTable.blockSignals(True)
        index = self.jTable.currentRow() + 1
        self.jTable.insertRow(index)
        [j.index(j.index() + 1) for j in self.jobList.iterate_list()
         if j.index() >= index]
        self.jTable.blockSignals(False)

    def jrem(self, *arg):
        self.jTable.blockSignals(True)
        index = self.jTable.selectedRanges()[0].bottomRow()
        if self.jobList.get(index) is not None:
            dialog = AlertWindow(self.topdir,
                                 "Can remove empty cells only. \
Remove name to delete")
            dialog.exec_()
        else:
            self.jTable.removeRow(index)
            [i.index(i.index() - 1)for i in self.jobList.iterate_list()
             if i.index() > index]
            j = self.jobList.get(index)
            if j is None:
                self.jsetsettings(self.jobList.get_anonym())
            else:
                self.jsetsettings(j.settings())
            self.jobLastSelected = index
        self.jTable.blockSignals(False)

    def jtableselchanged(self, *arg):
        lj = self.jobList.get(self.jobLastSelected)
        if lj is not None:
            lj.settings(self.jgetsettings())
        else:
            self.jobList.set_anonym(self.jgetsettings())

        if len(self.jTable.selectedIndexes()) == 0:
            self.jobLastSelected = -1
            self.jsetsettings(self.jobList.get_anonym())
        else:
            index = self.jTable.selectedIndexes()[0].row()
            j = self.jobList.get(index)
            if j is not None:
                self.jobLastSelected = j.index()
                self.jsetsettings(j.settings())
            else:
                self.jobLastSelected = -1
                self.jsetsettings(self.jobList.get_anonym())

    def jtableitemchanged(self, *arg):
        index = self.jTable.selectedIndexes()[0].row()
        text = self.jTable.item(index, 0).text()
        j = self.jobList.pop(index)
        if j is not None:
            if text == "" and j.name() != "":
                self.jobLastSelected = -1
                self.jsetsettings(self.jobList.get_anonym())
            else:
                j.name(text)
                self.jobLastSelected = j.index()
                self.jobList.push(j)
        else:
            if text != "":
                self.jobList.add(name=text,
                                 index=index,
                                 settings=self.jgetsettings())
                self.jobLastSelected = index

    def jgetcommsettings(self, *arg):
        d = {"gensec": {},
             "timesec": {}, }
        d["gensec"]["TITLE"] = self.TITLE.text()
        d["gensec"]["LSAV"] = self.LSAV.isChecked()
        d["gensec"]["LREC"] = self.LREC.isChecked()
        d["gensec"]["LCPUTM"] = self.LCPUTM.isChecked()
        d["gensec"]["ITEST"] = ITEST[self.ITEST.currentText()]
        d["gensec"]["TSOS"] = self.TSOS.value()
        d["gensec"]["TMAX"] = self.TMAX.value()
        d["timesec"]["MT"] = self.MT.value()
        d["timesec"]["NSHOT"] = self.NSHOT.value()
        d["timesec"]["TPS"] = self.TPS.text() if \
            self.TPS.text() != "" else None

        return d

    def jgetsettings(self, *arg):
        d = {"meshsec": {},
             "beamsec": {},
             "wakesec": {},
             "plotsec": {},
             "printsec": {}, }
        d["meshsec"]["DDR"] = self.DDR.text() if \
            self.DDR.text() != "" else None
        d["meshsec"]["RMARK"] = self.RMARK.text() if \
            self.RMARK.text() != "" else None
        d["meshsec"]["NR"] = self.NR.text() if \
            self.NR.text() != "" else None
        d["meshsec"]["NZ"] = self.NZ.text() if \
            self.NZ.text() != "" else None
        d["meshsec"]["RZ"] = self.RZ.text() if \
            self.RZ.text() != "" else None
        d["meshsec"]["DDZ"] = self.DDZ.text()
        d["beamsec"]["MROT"] = MROT[self.MROT.currentText()]
        d["beamsec"]["NBUNCH"] = self.NBUNCH.value()
        d["beamsec"]["ISIG"] = self.ISIG.value()
        d["beamsec"]["BSEP"] = self.BSEP.value()
        d["beamsec"]["SIG"] = self.SIG.text() if \
            self.SIG.text() != "" else None
        d["beamsec"]["RDRIVE"] = self.RDRIVE.text() if \
            self.RDRIVE.text() != "" else None
        d["wakesec"]["UBT"] = self.UBT.text() if \
            self.UBT.text() != "" else None
        d["wakesec"]["ZCF"] = self.ZCF.text() if \
            self.ZCF.text() != "" else None
        d["wakesec"]["ZCT"] = self.ZCT.text() if \
            self.ZCT.text() != "" else None
        d["wakesec"]["RWAK"] = self.RWAK.text() if \
            self.RWAK.text() != "" else None
        d["wakesec"]["ZSEP"] = self.ZSEP.value()
        d["wakesec"]["LCFRON"] = self.LCFRON.isChecked()
        d["wakesec"]["LCBACK"] = self.LCBACK.isChecked()
        d["wakesec"]["LCHIN"] = self.LCHIN.isChecked()
        d["wakesec"]["LNAPOLY"] = self.LNAPOLY.isChecked()
        d["wakesec"]["LNONAP"] = self.LNONAP.isChecked()
        d["wakesec"]["LCRBW"] = self.LCRBW.isChecked()
        d["plotsec"]["LCAVIN"] = self.LCAVIN.isChecked()
        d["plotsec"]["LCAVUS"] = self.LCAVUS.isChecked()
        d["plotsec"]["LPATH"] = self.LPATH.isChecked()
        d["plotsec"]["LPLW"] = self.LPLW.isChecked()
        d["plotsec"]["LPLWL"] = self.LPLWL.isChecked()
        d["plotsec"]["LPLWA"] = self.LPLWA.isChecked()
        d["plotsec"]["LPLWT"] = self.LPLWT.isChecked()
        d["plotsec"]["LFFT"] = self.LFFT.isChecked()
        d["plotsec"]["LFFTL"] = self.LFFTL.isChecked()
        d["plotsec"]["LFFTA"] = self.LFFTA.isChecked()
        d["plotsec"]["LFFTT"] = self.LFFTT.isChecked()
        d["plotsec"]["LINTZ"] = self.LINTZ.isChecked()
        d["plotsec"]["LSPEC"] = self.LSPEC.isChecked()
        d["plotsec"]["LWNDW"] = self.LWNDW.isChecked()
        d["plotsec"]["LPLE"] = self.LPLE.isChecked()
        d["plotsec"]["LPLC"] = self.LPLC.isChecked()
        d["plotsec"]["LPALL"] = self.LPALL.isChecked()
        d["plotsec"]["ALPHA"] = self.ALPHA.value()
        d["plotsec"]["EXPFAC"] = self.EXPFAC.value()
        d["plotsec"]["CUTOFF"] = self.CUTOFF.value()
        d["plotsec"]["NPLOT"] = self.NPLOT.value()
        d["plotsec"]["NWFUN"] = NWFUN[self.NWFUN.currentText()]
        d["printsec"]["LPRW"] = self.LPRW.isChecked()
        d["printsec"]["LMATPR"] = self.LMATPR.isChecked()
        d["printsec"]["LSVW"] = self.LSVW.isChecked()
        d["printsec"]["LSVWL"] = self.LSVWL.isChecked()
        d["printsec"]["LSVWA"] = self.LSVWA.isChecked()
        d["printsec"]["LSVWT"] = self.LSVWT.isChecked()
        d["printsec"]["LSVF"] = self.LSVF.isChecked()

        return d

    def jsetcommsettings(self, d):
        self.TITLE.setText(d["gensec"]["TITLE"])
        self.LSAV.setChecked(d["gensec"]["LSAV"])
        self.LREC.setChecked(d["gensec"]["LREC"])
        self.LCPUTM.setChecked(d["gensec"]["LCPUTM"])
        self.ITEST.setCurrentText(RITEST[d["gensec"]["ITEST"]])
        self.TSOS.setValue(d["gensec"]["TSOS"])
        self.TMAX.setValue(d["gensec"]["TMAX"])
        self.MT.setValue(d["timesec"]["MT"])
        self.NSHOT.setValue(d["timesec"]["NSHOT"])
        if d["timesec"]["TPS"] is None:
            self.TPS.setText("")
        else:
            self.TPS.setText(d["timesec"]["TPS"])

    def jsetsettings(self, d):
        if d["meshsec"]["DDR"] is None:
            self.DDR.setText("")
        else:
            self.DDR.setText(d["meshsec"]["DDR"])
        if d["meshsec"]["RMARK"] is None:
            self.RMARK.setText("")
        else:
            self.RMARK.setText(d["meshsec"]["RMARK"])
        if d["meshsec"]["NR"] is None:
            self.NR.setText("")
        else:
            self.NR.setText(d["meshsec"]["NR"])
        if d["meshsec"]["NZ"] is None:
            self.NZ.setText("")
        else:
            self.NZ.setText(d["meshsec"]["NZ"])
        if d["meshsec"]["RZ"] is None:
            self.RZ.setText("")
        else:
            self.RZ.setText(d["meshsec"]["RZ"])
        self.DDZ.setText(d["meshsec"]["DDZ"])
        self.MROT.setCurrentText(RMROT[d["beamsec"]["MROT"]])
        self.NBUNCH.setValue(d["beamsec"]["NBUNCH"])
        self.ISIG.setValue(d["beamsec"]["ISIG"])
        self.BSEP.setValue(d["beamsec"]["BSEP"])
        if d["beamsec"]["SIG"] is None:
            self.SIG.setText("")
        else:
            self.SIG.setText(d["beamsec"]["SIG"])
        if d["beamsec"]["RDRIVE"] is None:
            self.RDRIVE.setText("")
        else:
            self.RDRIVE.setText(d["beamsec"]["RDRIVE"])
        if d["wakesec"]["UBT"] is None:
            self.UBT.setText("")
        else:
            self.UBT.setText(d["wakesec"]["UBT"])
        if d["wakesec"]["ZCF"] is None:
            self.ZCF.setText("")
        else:
            self.ZCF.setText(d["wakesec"]["ZCF"])
        if d["wakesec"]["ZCT"] is None:
            self.ZCT.setText("")
        else:
            self.ZCT.setText(d["wakesec"]["ZCT"])
        if d["wakesec"]["RWAK"] is None:
            self.RWAK.setText("")
        else:
            self.RWAK.setText(d["wakesec"]["RWAK"])
        self.ZSEP.setValue(d["wakesec"]["ZSEP"])
        self.LCFRON.setChecked(d["wakesec"]["LCFRON"])
        self.LCBACK.setChecked(d["wakesec"]["LCBACK"])
        self.LCHIN.setChecked(d["wakesec"]["LCHIN"])
        self.LNAPOLY.setChecked(d["wakesec"]["LNAPOLY"])
        self.LNONAP.setChecked(d["wakesec"]["LNONAP"])
        self.LCRBW.setChecked(d["wakesec"]["LCRBW"])
        self.LCAVIN.setChecked(d["plotsec"]["LCAVIN"])
        self.LCAVUS.setChecked(d["plotsec"]["LCAVUS"])
        self.LPATH.setChecked(d["plotsec"]["LPATH"])
        self.LPLW.setChecked(d["plotsec"]["LPLW"])
        self.LPLWL.setChecked(d["plotsec"]["LPLWL"])
        self.LPLWA.setChecked(d["plotsec"]["LPLWA"])
        self.LPLWT.setChecked(d["plotsec"]["LPLWT"])
        self.LFFT.setChecked(d["plotsec"]["LFFT"])
        self.LFFTL.setChecked(d["plotsec"]["LFFTL"])
        self.LFFTA.setChecked(d["plotsec"]["LFFTA"])
        self.LFFTT.setChecked(d["plotsec"]["LFFTT"])
        self.LINTZ.setChecked(d["plotsec"]["LINTZ"])
        self.LSPEC.setChecked(d["plotsec"]["LSPEC"])
        self.LWNDW.setChecked(d["plotsec"]["LWNDW"])
        self.LPLE.setChecked(d["plotsec"]["LPLE"])
        self.LPLC.setChecked(d["plotsec"]["LPLC"])
        self.LPALL.setChecked(d["plotsec"]["LPALL"])
        self.ALPHA.setValue(d["plotsec"]["ALPHA"])
        self.EXPFAC.setValue(d["plotsec"]["EXPFAC"])
        self.CUTOFF.setValue(d["plotsec"]["CUTOFF"])
        self.NPLOT.setValue(d["plotsec"]["NPLOT"])
        self.NWFUN.setCurrentText(RNWFUN[d["plotsec"]["NWFUN"]])
        self.LPRW.setChecked(d["printsec"]["LPRW"])
        self.LMATPR.setChecked(d["printsec"]["LMATPR"])
        self.LSVW.setChecked(d["printsec"]["LSVW"])
        self.LSVWL.setChecked(d["printsec"]["LSVWL"])
        self.LSVWA.setChecked(d["printsec"]["LSVWA"])
        self.LSVWT.setChecked(d["printsec"]["LSVWT"])
        self.LSVF.setChecked(d["printsec"]["LSVF"])

    def pupload(self, *arg):
        file, type = QtWidgets.QFileDialog.getOpenFileName(
            self,
            "Open File",
            self.lastdir,
            "TopDraw Files (*.top);;All Files (*)"
        )
        if isfile(file):
            self.lastdir = path.dirname(file)
            self.data = prs.top_parse(file)
            self.pList.clear()
            for i in self.data:
                title = i.title_dump() if i.title_dump() != "" \
                    else "No name avaliable"
                self.pList.addItem(title)

    def pdownload(self, *arg):
        file, type = QtWidgets.QFileDialog.getSaveFileName(
            self,
            "Save File",
            self.lastdir,
            "TOP Files (*.top);;All Files (*)"
        )
        if file != "":
            outputfile = path.join(self.tempDir, "workfile.top")
            if isfile(outputfile):
                try:
                    copyfile(outputfile, file)
                except IOError:
                    dialog = AlertWindow(self.topdir, "Cannot open file")
                    dialog.exec_()
            else:
                dialog = AlertWindow(self.topdir, "There is no top \
file to download")
                dialog.exec_()

    def plistselect(self, index):
        data = self.data[index].data_dump()
        self.pCanvas.axes.clear()
        for style in data.keys():
            for d in data[style]:
                self.pCanvas.axes.plot(d[:, 0],
                                       d[:, 1],
                                       linestyle=style,
                                       color="black")
        self.pCanvas.axes.set_title(self.data[index].title_dump())
        self.pCanvas.axes.set_xlabel(self.data[index].xlab_dump())
        self.pCanvas.axes.set_ylabel(self.data[index].ylab_dump())
        self.pCanvas.axes.grid()
        self.pInfo.setText("\n".join(self.data[index].description_dump()))
        legends = []
        for style, label in self.data[index].legends_dump().items():
            legends.append(mlines.Line2D([], [],
                                         color='black',
                                         linestyle=style,
                                         label=label))
        if len(legends) > 0:
            self.pCanvas.axes.legend(handles=legends)
        self.pCanvas.draw()

    def ssetExec(self, *arg):
        file, type = QtWidgets.QFileDialog.getOpenFileName(
            self,
            "ABCI binary",
            self.lastdir,
            "All Files (*)"
        )
        if isfile(file) and access(file, X_OK):
            self.execLocation.setText(file)
            self.lastdir = path.dirname(file)

    def closeEvent(self, *arg):
        if self.tempDir is not None:
            try:
                rmtree(self.tempDir)
            except FileNotFoundError:
                pass


class MplCanvas(FigureCanvas):
    def __init__(self, parent=None, width=5, height=4, dpi=100):
        self.fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = self.fig.add_subplot(111)

        FigureCanvas.__init__(self, self.fig)
        self.setParent(parent)

        FigureCanvas.setSizePolicy(self,
                                   QtWidgets.QSizePolicy.Expanding,
                                   QtWidgets.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)


class AlertWindow(QtWidgets.QDialog):
    def __init__(self, topdir, message):
        super(AlertWindow, self).__init__()
        uic.loadUi(path.join(topdir, "ui", "alert.ui"), self)
        self.textBrowser.setText(message)


class ProgressWindow(QtWidgets.QDialog):
    def __init__(self, topdir, process):
        super(ProgressWindow, self).__init__()
        uic.loadUi(path.join(topdir, "ui", "progress.ui"), self)
        self.topdir = topdir
        self.interrupt.clicked.connect(lambda: process.terminate())
        self.thread = CloneThread(process)
        self.thread.newline.connect(self.newline)
        self.thread.eof.connect(self.close)
        self.thread.start()
        self.closeEvent = lambda x: process.terminate()

    def newline(self, msg):
        self.output.addItem(msg)


class CloneThread(QtCore.QThread):
    newline = QtCore.pyqtSignal('PyQt_PyObject')
    eof = QtCore.pyqtSignal('PyQt_PyObject')

    def __init__(self, process):
        QtCore.QThread.__init__(self)
        self.proc = process

    def run(self):
        out = None
        while self.proc.poll() is None:
            out = self.proc.stdout.readline().decode().strip()
            self.newline.emit(out)
        self.eof.emit(out)



from setuptools import setup, find_packages
from codecs import open
from os import path
import pytopplot.version
import sys


here = path.abspath(path.dirname(__file__))

if not sys.version_info[0] == 3 and \
       sys.version_info[1] >= 5:
    sys.exit("Sorry, Python 3.5 or higher only")

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='pytopplot',
    version=pytopplot.version.version,
    description='GUI for ABCI program',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://bitbucket.com/seregaxvm/pytopplot',
    author='S.V. Matsievskiy',
    author_email='matsievskiysv@gmail.com',
    license='GPLv3+',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: X11 Applications :: Qt',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public \
License v3 or later (GPLv3+)',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3.5',
        'Topic :: Scientific/Engineering :: Physics',
    ],
    keywords='wakefields accelerators particles GUI',
    packages=find_packages(exclude=["*.tests",
                                    "*.tests.*",
                                    "tests.*",
                                    "tests"]),
    include_package_data=True,
    install_requires=['numpy', 'PyQt5', 'matplotlib', 'PyYAML'],
    extras_require={
        'dev': ['check-manifest'],
    },
    entry_points={
        'console_scripts': ['pytopplot = pytopplot.__main__:main']
    }
)
